@@ -16,11 +16,11 @@ Suricata::Monitoring - LibreNMS JSON SNMP extend and Nagios style check for Suri
 
 =head1 VERSION
 
-Version 0.0.1
+Version 0.1.0
 
 =cut
 
-our $VERSION = '0.0.1';
+our $VERSION = '0.1.0';
 
 =head1 SYNOPSIS
 
@@ -78,7 +78,8 @@ The only must have is 'files'.
       - Default :: .1
     
     - files :: A hash with the keys being the instance name and the values
-      being the Eve files to read.
+      being the Eve files to read. ".total" is not a valid instance name.
+      Similarly anything starting with a "." should be considred reserved.
 
     my $args = {
         mode               => 'librenms',
@@ -154,6 +155,10 @@ sub new {
 		$self->{files} = $args{files};
 	}
 
+	if ( defined( $self->{files}{'.total'} ) ) {
+		confess('".total" is not a valid instance name');
+	}
+
 	# pull in cache dir location
 	if ( !defined( $args{cache_dir} ) ) {
 		$args{cache_dir} = '/var/cache/suricata-monitoring/';
@@ -184,7 +189,14 @@ sub run {
 	my $self = $_[0];
 
 	# this will be returned
-	my $to_return = { data => {}, version => 1, error => '0', errorString => '', alert => '0', alertString => '' };
+	my $to_return = {
+		data        => { '.total' => {} },
+		version     => 1,
+		error       => '0',
+		errorString => '',
+		alert       => '0',
+		alertString => ''
+	};
 
 	my $previous;
 	my $previous_file = $self->{cache_dir} . '/stats.json';
@@ -213,6 +225,9 @@ sub run {
 	my $current_till;
 	foreach my $instance (@instances) {
 
+		# ends processing for this file
+		my $process_it = 1;
+
 		# open the file for reading it backwards
 		my $bw;
 		eval {
@@ -225,13 +240,14 @@ sub run {
 				$to_return->{errorString} = $to_return->{errorString} . "\n";
 			}
 			$to_return->{errorString} = $to_return->{errorString} . $instance . ': ' . $@;
+			$process_it = 0;
 		}
 
-		#
-		my $process_it = 1;
-		my $line       = $bw->readline;
-		my $found;
-
+		# get the first line, if possible
+		my $line;
+		if ($process_it) {
+			$line = $bw->readline;
+		}
 		while ( $process_it
 			&& defined($line) )
 		{
@@ -240,17 +256,20 @@ sub run {
 				my $timestamp = $json->{timestamp};
 
 				# if current till is not set, set it
-				if ( !defined($current_till) ) {
+				if (  !defined($current_till)
+					&& defined($timestamp)
+					&& $timestamp =~ /^[0-9]+\-[0-9]+\-[0-9]+T[0-9]+\:[0-9]+\:[0-9\.]+[\-\+][0-9]+/ )
+				{
 
 					# get the number of hours
 					my $hours = $timestamp;
-					$hours =~ s/.*[-+]//g;
+					$hours =~ s/.*[\-\+]//g;
 					$hours =~ s/^0//;
 					$hours =~ s/[0-9][0-9]$//;
 
 					# get the number of minutes
 					my $minutes = $timestamp;
-					$minutes =~ s/.*[-+]//g;
+					$minutes =~ s/.*[\-\+]//g;
 					$minutes =~ s/^[0-9][0-9]//;
 
 					my $second_diff = ( $minutes * 60 ) + ( $hours * 60 * 60 );
@@ -266,7 +285,7 @@ sub run {
 				my $t = Time::Piece->strptime( $timestamp, '%Y-%m-%dT%H:%M:%S' );
 
 				# stop process further lines as we've hit the oldest we care about
-				if ( $t->epoch < $current_till ) {
+				if ( $t->epoch <= $current_till ) {
 					$process_it = 0;
 				}
 
@@ -303,24 +322,47 @@ sub run {
 						dec_tcp          => $json->{stats}{decoder}{tcp},
 						dec_avg_pkt_size => $json->{stats}{decoder}{avg_pkt_size},
 						dec_max_pkt_size => $json->{stats}{decoder}{max_pkt_size},
-						f_tcp            => $json->{stats}{flow}{tcp},
-						f_udp            => $json->{stats}{flow}{udp},
-						f_icmpv4         => $json->{stats}{flow}{icmpv4},
-						f_icmpv6         => $json->{stats}{flow}{icmpv6},
-						f_memuse         => $json->{stats}{flow}{memuse},
-						ftp_memuse       => $json->{stats}{ftp}{memuse},
-						http_memuse      => $json->{stats}{http}{memuse},
-						tcp_memuse       => $json->{stats}{tcp}{memuse},
-						tcp_reass_memuse => $json->{stats}{tcp}{reassembly_memuse},
-						alert            => 0,
-						alert_string     => '',
+						dec_chdlc          => $json->{stats}{decoder}{chdlc},
+						dec_ethernet       => $json->{stats}{decoder}{ethernet},
+						dec_geneve         => $json->{stats}{decoder}{geneve},
+						dec_ieee8021ah     => $json->{stats}{decoder}{ieee8021ah},
+						dec_ipv4_in_ipv6   => $json->{stats}{decoder}{ipv6_in_ipv6},
+						dec_mx_mac_addrs_d => $json->{stats}{decoder}{max_mac_addrs_dst},
+						dec_mx_mac_addrs_s => $json->{stats}{decoder}{max_mac_addrs_src},
+						dec_mpls           => $json->{stats}{decoder}{mpls},
+						dec_ppp            => $json->{stats}{decoder}{ppp},
+						dec_pppoe          => $json->{stats}{decoder}{pppoe},
+						dec_raw            => $json->{stats}{decoder}{raw},
+						dec_sctp           => $json->{stats}{decoder}{sctp},
+						dec_sll            => $json->{stats}{decoder}{sll},
+						dec_teredo         => $json->{stats}{decoder}{teredo},
+						dec_too_many_layer => $json->{stats}{decoder}{too_many_layers},
+						dec_vlan           => $json->{stats}{decoder}{vlan},
+						dec_vlan_qinq      => $json->{stats}{decoder}{vlan_qinq},
+						dec_vntag          => $json->{stats}{decoder}{vntag},
+						dec_vxlan          => $json->{stats}{decoder}{vxlan},
+						f_tcp              => $json->{stats}{flow}{tcp},
+						f_udp              => $json->{stats}{flow}{udp},
+						f_icmpv4           => $json->{stats}{flow}{icmpv4},
+						f_icmpv6           => $json->{stats}{flow}{icmpv6},
+						f_memuse           => $json->{stats}{flow}{memuse},
+						ftp_memuse         => $json->{stats}{ftp}{memuse},
+						http_memuse        => $json->{stats}{http}{memuse},
+						tcp_memuse         => $json->{stats}{tcp}{memuse},
+						tcp_reass_memuse   => $json->{stats}{tcp}{reassembly_memuse},
+						alert              => 0,
+						alertString        => '',
 					};
 
-					foreach my $flow_key ( keys( %{ $json->{stats}{app_layer}{flows} } ) ) {
-						$new_stats->{ 'af_' . $flow_key } = $json->{stats}{app_layer}{flows}{$flow_key};
+					foreach my $flow_key ( keys( %{ $json->{stats}{app_layer}{flow} } ) ) {
+						my $new_key = $flow_key;
+						$new_key =~ s/\-/_/g;
+						$new_stats->{ 'af_' . $new_key } = $json->{stats}{app_layer}{flow}{$flow_key};
 					}
 					foreach my $tx_key ( keys( %{ $json->{stats}{app_layer}{tx} } ) ) {
-						$new_stats->{ 'at_' . $tx_key } = $json->{stats}{app_layer}{tx}{$tx_key};
+						my $new_key = $tx_key;
+						$new_key =~ s/\-/_/g;
+						$new_stats->{ 'at_' . $new_key } = $json->{stats}{app_layer}{tx}{$tx_key};
 					}
 
 					# some this is a bit variable as to which will be present based on the system
@@ -482,22 +524,56 @@ sub run {
 						}
 					}
 
+					# add stuff to .total
+					my @intance_keys = keys( %{$new_stats} );
+					foreach my $total_key (@intance_keys) {
+						if ( $total_key ne 'alertString' ) {
+							if ( !defined( $to_return->{data}{'.total'}{$total_key} ) ) {
+								$to_return->{data}{'.total'}{$total_key} = $new_stats->{$total_key};
+							}
+							else {
+								$to_return->{data}{'.total'}{$total_key}
+									= $to_return->{data}{'.total'}{$total_key} + $new_stats->{$total_key};
+							}
+						}
+					}
+
 					$to_return->{data}{$instance} = $new_stats;
 				}
 
 			};
+
+			# get the next line
+			$line = $bw->readline;
 		}
 
 	}
 
+	$to_return->{data}{'.total'}{alert} = $to_return->{'alert'};
+
 	# join any found alerts into the string
-	$to_return->{alertsString} = join( "\n", @alerts );
+	$to_return->{alertString} = join( "\n", @alerts );
+
+	# compute percents for .total
+	if ( $to_return->{data}{'.total'}{packet_delta} != 0 ) {
+		$to_return->{data}{'.total'}{drop_percent}
+			= ( $to_return->{data}{'.total'}{drop_delta} / $to_return->{data}{'.total'}{packet_delta} ) * 100;
+		$to_return->{data}{'.total'}{drop_percent} = sprintf( '%0.5f', $to_return->{data}{'.total'}{ifdrop_percent} );
+
+		$to_return->{data}{'.total'}{ifdrop_percent}
+			= ( $to_return->{data}{'.total'}{ifdrop_delta} / $to_return->{data}{'.total'}{packet_delta} ) * 100;
+		$to_return->{data}{'.total'}{ifdrop_percent} = sprintf( '%0.5f', $to_return->{data}{'.total'}{ifdrop_percent} );
+
+		$to_return->{data}{'.total'}{error_percent}
+			= ( $to_return->{data}{'.total'}{error_delta} / $to_return->{data}{'.total'}{packet_delta} ) * 100;
+		$to_return->{data}{'.total'}{error_percent} = sprintf( '%0.5f', $to_return->{data}{'.total'}{error_percent} );
+	}
 
 	# write the cache file on out
 	eval {
 		my $new_cache = encode_json($to_return);
 		open( my $fh, '>', $previous_file );
-		print $fh $new_cache."\n";
+		print $fh $new_cache . "\n";
 		close($fh);
 	};
 	if ($@) {
@@ -555,6 +631,80 @@ sub print_output {
 	}
 }
 
+=head1 LibreNMS HASH
+
+    + $hash{'alert'} :: Alert status.
+      - 0 :: OK
+      - 1 :: WARNING
+      - 2 :: CRITICAL
+      - 3 :: UNKNOWN
+    
+    + $hash{'alertString'} :: A string describing the alert. Defaults to
+      '' if there is no alert.
+    
+    + $hash{'error'} :: A integer representing a error. '0' represents
+      everything is fine.
+    
+    + $hash{'errorString'} :: A string description of the error.
+    
+    + $hash{'data'}{$instance} :: Values migrated from the
+      instance. *_delta values are created via computing the difference
+      from the previously saved info. *_percent is based off of the delta
+      in question over the packet delta. Delta are created for packet,
+      drop, ifdrop, and error. Percents are made for drop, ifdrop, and
+      error.
+    
+    + $hash{'data'}{'.total'} :: Total values of from all the
+      intances. Any percents will be recomputed.
+    
+
+    The stat keys are migrated as below.
+    
+    uptime           => $json->{stats}{uptime},
+    packets          => $json->{stats}{capture}{kernel_packets},
+    dropped          => $json->{stats}{capture}{kernel_drops},
+    ifdropped        => $json->{stats}{capture}{kernel_ifdrops},
+    errors           => $json->{stats}{capture}{errors},
+    bytes            => $json->{stats}{decoder}{bytes},
+    dec_packets      => $json->{stats}{decoder}{pkts},
+    dec_invalid      => $json->{stats}{decoder}{invalid},
+    dec_ipv4         => $json->{stats}{decoder}{ipv4},
+    dec_ipv6         => $json->{stats}{decoder}{ipv6},
+    dec_udp          => $json->{stats}{decoder}{udp},
+    dec_tcp          => $json->{stats}{decoder}{tcp},
+    dec_avg_pkt_size => $json->{stats}{decoder}{avg_pkt_size},
+    dec_max_pkt_size => $json->{stats}{decoder}{max_pkt_size},
+    dec_chdlc          => $json->{stats}{decoder}{chdlc},
+    dec_ethernet       => $json->{stats}{decoder}{ethernet},
+    dec_geneve         => $json->{stats}{decoder}{geneve},
+    dec_ieee8021ah     => $json->{stats}{decoder}{ieee8021ah},
+    dec_ipv4_in_ipv6   => $json->{stats}{decoder}{ipv6_in_ipv6},
+    dec_mx_mac_addrs_d => $json->{stats}{decoder}{max_mac_addrs_dst},
+    dec_mx_mac_addrs_s => $json->{stats}{decoder}{max_mac_addrs_src},
+    dec_mpls           => $json->{stats}{decoder}{mpls},
+    dec_ppp            => $json->{stats}{decoder}{ppp},
+    dec_pppoe          => $json->{stats}{decoder}{pppoe},
+    dec_raw            => $json->{stats}{decoder}{raw},
+    dec_sctp           => $json->{stats}{decoder}{sctp},
+    dec_sll            => $json->{stats}{decoder}{sll},
+    dec_teredo         => $json->{stats}{decoder}{teredo},
+    dec_too_many_layer => $json->{stats}{decoder}{too_many_layers},
+    dec_vlan           => $json->{stats}{decoder}{vlan},
+    dec_vlan_qinq      => $json->{stats}{decoder}{vlan_qinq},
+    dec_vntag          => $json->{stats}{decoder}{vntag},
+    dec_vxlan          => $json->{stats}{decoder}{vxlan},
+    f_tcp              => $json->{stats}{flow}{tcp},
+    f_udp              => $json->{stats}{flow}{udp},
+    f_icmpv4           => $json->{stats}{flow}{icmpv4},
+    f_icmpv6           => $json->{stats}{flow}{icmpv6},
+    f_memuse           => $json->{stats}{flow}{memuse},
+    ftp_memuse         => $json->{stats}{ftp}{memuse},
+    http_memuse        => $json->{stats}{http}{memuse},
+    tcp_memuse         => $json->{stats}{tcp}{memuse},
+    tcp_reass_memuse   => $json->{stats}{tcp}{reassembly_memuse},
+    af_*               => $json->{stats}{app_layer}{flow}{*}
+    at_*               => $json->{stats}{app_layer}{tx}{*}
+
 =head1 AUTHOR
 
 Zane C. Bowers-Hadley, C<< <vvelox at vvelox.net> >>
